@@ -2,7 +2,7 @@ package MVC::Neaf::Exception;
 
 use strict;
 use warnings;
-our $VERSION = 0.2203;
+our $VERSION = 0.23;
 
 =head1 NAME
 
@@ -50,6 +50,8 @@ otherwise is reset to 500 and reason is updated.
 
 =item * -reason - details about what happened
 
+=item * -headers - array or hash of headers, just like that of a normal reply.
+
 =item * -location - indicates a redirection
 
 =item * -sudden - this was not an expected error (die 404 or redirect)
@@ -153,7 +155,7 @@ sub make_reply {
         -status   => $self->{-status},
         -content  => "See $self->{-location}\n",
         -type     => 'text/plain; charset=utf8',
-        -headers  => [ location => $self->{-location} ],
+        -headers  => [ Location => $self->{-location}, @{ $self->{-headers} || [] } ],
     } if ($self->{-location});
 
     my $req_id = $req->id;
@@ -161,6 +163,7 @@ sub make_reply {
         -status   => $self->{-status},
         -type     => 'application/json',
         -content  => qq({"error":"$self->{-status}","req_id":"$req_id"}),
+        -headers  => $self->{-headers},
     };
 };
 
@@ -203,13 +206,15 @@ sub TO_JSON {
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2016-2017 Konstantin S. Uvarin C<khedin@cpan.org>.
+This module is part of L<MVC::Neaf> suite.
+
+Copyright 2016-2018 Konstantin S. Uvarin C<khedin@cpan.org>.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
 by the Free Software Foundation; or the Artistic License.
 
-See http://dev.perl.org/licenses/ for more information.
+See L<http://dev.perl.org/licenses/> for more information.
 
 =cut
 
