@@ -3,7 +3,7 @@ package MVC::Neaf::Request;
 use strict;
 use warnings;
 
-our $VERSION = 0.2203;
+our $VERSION = 0.23;
 
 =head1 NAME
 
@@ -29,8 +29,8 @@ Here's a brief overview of what a Neaf request returns:
     $req->hostname     = server.name
     $req->port         = 1337
     $req->path         = /mathing/route/some/more/slashes
-    $req->script_name  = /mathing/route
-    $req->path_info    = some/more/slashes
+    $req->prefix       = /mathing/route
+    $req->postfix      = some/more/slashes
 
     # params and cookies require a regex
     $req->param( foo => '\d+' ) = 1
@@ -205,6 +205,18 @@ sub path {
     return $self->{path} ||= $self->do_get_path;
 };
 
+=head2 route()
+
+A L<MVC::Neaf::Route> object that this request is being dispatched to,
+or the root of the routing tree if none yet.
+
+=cut
+
+sub route {
+    my $self = shift;
+    return $self->{route};
+};
+
 =head2 set_path()
 
     $req->set_path( $new_path )
@@ -230,7 +242,7 @@ sub set_path {
     $self;
 };
 
-=head2 script_name()
+=head2 prefix()
 
 The part of the request that matched the route to the
 application being executed.
@@ -242,13 +254,14 @@ Not available before routing was applied to request.
 
 =cut
 
-sub script_name {
+sub prefix {
     my $self = shift;
 
-    carp "NEAF: script_name call before routing was applied is DEPRECATED"
-        unless $self->{endpoint};
+    # TODO kill in 0.30
+    carp "NEAF: prefix() call before routing was applied is DEPRECATED: "
+        unless $self->{route} && $self->{route}->path =~ m,^(?:/|$),;
 
-    return $self->{script_name} ||= $self->path;
+    return $self->{prefix} ||= $self->path;
 };
 
 =head2 get_url_base()
@@ -324,7 +337,7 @@ sub get_url_full {
     return $self->get_url_base . $self->get_url_rel(@_);
 };
 
-=head2 path_info()
+=head2 postfix()
 
 Returns the part of URI path beyond what matched the application's path.
 
@@ -339,34 +352,36 @@ B<[EXPERIMENTAL]> This part of API is undergoing changes.
 
 =cut
 
-sub path_info {
+sub postfix {
     my ($self) = @_;
 
-    return $self->{path_info};
+    return $self->{postfix};
 };
 
-=head2 path_info_split()
+=head2 splat()
 
 Return a list of matched capture groups found in path_info_regex, if any.
 
-B<[EXPERIMENTAL]> Name and meaning subject to change.
-
 =cut
 
-sub path_info_split {
+sub splat {
     my $self = shift;
 
     return @{ $self->{path_info_split} || [] };
 };
 
-
+# This is somewhat ugly.
+# When a route is matched, we'll store it
+#    and _also_ the leftovers of regex splitting that happens around that moment
 sub _import_route {
     my ($self, $route, $path, $path_info, $tail) = @_;
 
-    $self->{endpoint}        = $route;
-    $self->{script_name}  = $path;
-    $self->{path_info}    = $path_info;
-    $self->{path_info_split}   = $tail;
+    $self->{route}        = $route;
+    if (defined $path) {
+        $self->{prefix}  = $path;
+        $self->{postfix}    = $path_info;
+        $self->{path_info_split}   = $tail;
+    };
 
     return $self;
 };
@@ -395,9 +410,9 @@ sub set_path_info {
     # CANONIZE
     $path_info =~ s#^/+##;
 
-    $self->{path_info} = $path_info;
+    $self->{postfix} = $path_info;
     $self->{path} = $self->script_name
-        .(length $self->{path_info} ? "/$self->{path_info}" : '');
+        .(length $self->{postfix} ? "/$self->{postfix}" : '');
 
     return $self;
 };
@@ -447,7 +462,7 @@ See L<MVC::Neaf::X::Form::Wildcard>.
 sub param {
     my ($self, $name, $regex, $default) = @_;
 
-    $regex ||= $self->{endpoint}{param_regex}{$name};
+    $regex ||= $self->{route}{param_regex}{$name};
 
     $self->_croak( "NEAF: param(): a validation regex is REQUIRED" )
         unless defined $regex;
@@ -538,7 +553,7 @@ Please be careful when upgrading.
 sub multi_param {
     my ($self, $name, $regex) = @_;
 
-    $regex ||= $self->{endpoint}{param_regex}{$name};
+    $regex ||= $self->{route}{param_regex}{$name};
     $self->_croak( "validation regex is REQUIRED" )
         unless defined $regex;
 
@@ -609,7 +624,7 @@ sub form {
     my ($self, $validator) = @_;
 
     if (!ref $validator) {
-        $validator = $self->{_neaf} && $self->{_neaf}->get_form( $validator )
+        $validator = $self->{route}->get_form( $validator )
             || $self->_croak("Unknown form name $validator");
     };
 
@@ -1065,8 +1080,8 @@ sub dump {
     $raw{header_in} = $self->header_in->as_string;
     $self->get_cookie( noexist => '' ); # warm up cookie cache
     $raw{cookie_in} = $self->{neaf_cookie_in};
-    $raw{path_info} = $self->{path_info}
-        if defined $self->{path_info};
+    $raw{path_info} = $self->{postfix}
+        if defined $self->{postfix};
 
     return \%raw;
 };
@@ -1310,6 +1325,10 @@ sub reply {
 
 sub _set_reply {
     my ($self, $data) = @_;
+
+    croak "Bareword forbidden in reply"
+        unless ref $data and UNIVERSAL::isa($data, 'HASH');
+
     $self->{response}{ret} = $data;
     return $self;
 }
@@ -1551,7 +1570,7 @@ sub log_error {
 sub _where {
     my $self = shift;
 
-    return $self->{script_name} || "pre_route";
+    return $self->{prefix} || "pre_route";
 };
 
 sub _log_mark {
@@ -1701,6 +1720,33 @@ Please keep an eye on C<Changes> though.
 
 Here are these methods, for the sake of completeness.
 
+=cut
+
+=head2 script_name
+
+See L</prefix>
+
+=head2 path_info
+
+See L</postfix>.
+
+=head2 path_info_split
+
+See L</prefix>
+
+These three will start warning in 0.30 and will be removed in 0.40
+
+=cut
+
+# TODO 0.30 start warning "deprecated"
+{
+    no warnings 'once'; ## no critic
+    use warnings FATAL => 'redefine';
+    *script_name = \&prefix;
+    *path_info = \&postfix;
+    *path_info_split = \&splat;
+}
+
 =head2 header_in_keys ()
 
 Return all keys in header_in object as a list.
@@ -1743,8 +1789,8 @@ B<[DEPRECATED]> This function was added prematurely and shall not be used.
 sub endpoint_origin {
     my $self = shift;
 
-    return '(unspecified file):0' unless $self->{endpoint}{caller};
-    return join " line ", @{ $self->{endpoint}{caller} }[1,2];
+    return '(unspecified file):0' unless $self->{route}{caller};
+    return join " line ", @{ $self->{route}{caller} }[1,2];
 };
 
 =head2 set_full_path( ... )
@@ -1783,21 +1829,21 @@ sub set_full_path {
 
     # CANONIZE
     $script_name =~ s#^/*#/#;
-    $self->{script_name} = $script_name;
+    $self->{prefix} = $script_name;
 
     if (defined $path_info) {
         # Make sure path_info always has a slash if nonempty
         $path_info =~ s#^/+##;
-        $self->{path_info} = Encode::is_utf8($path_info)
+        $self->{postfix} = Encode::is_utf8($path_info)
                 ? $path_info
                 : decode_utf8(uri_unescape($path_info));
-    } elsif (!defined $self->{path_info}) {
-        $self->{path_info} = '';
+    } elsif (!defined $self->{postfix}) {
+        $self->{postfix} = '';
     };
-    # assert $self->{path_info} is defined by now
+    # assert $self->{postfix} is defined by now
 
-    $self->{path} = "$self->{script_name}"
-        .(length $self->{path_info} ? "/$self->{path_info}" : '');
+    $self->{path} = "$self->{prefix}"
+        .(length $self->{postfix} ? "/$self->{postfix}" : '');
     return $self;
 };
 
@@ -1857,7 +1903,9 @@ sub get_default {
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2016-2017 Konstantin S. Uvarin C<khedin@cpan.org>.
+This module is part of L<MVC::Neaf> suite.
+
+Copyright 2016-2018 Konstantin S. Uvarin C<khedin@cpan.org>.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
