@@ -3,7 +3,7 @@ package MVC::Neaf::Request::Apache2;
 use strict;
 use warnings;
 
-our $VERSION = 0.2203;
+our $VERSION = 0.23;
 
 =head1 NAME
 
@@ -293,7 +293,7 @@ sub handler : method {
     if (!$MVC::Neaf::Request::query_allowed{ $r->method }) {
         $r->args('');
     };
-    my $reply = MVC::Neaf->handle_request( $self );
+    my $reply = MVC::Neaf::neaf()->handle_request( $self );
 
     return Apache2::Const::OK();
 };
@@ -313,4 +313,18 @@ sub failed_startup {
        return %fail_apache ? \%fail_apache : ();
 };
 
+=head1 LICENSE AND COPYRIGHT
+
+This module is part of L<MVC::Neaf> suite.
+
+Copyright 2016-2018 Konstantin S. Uvarin C<khedin@cpan.org>.
+
+This program is free software; you can redistribute it and/or modify it
+under the terms of either: the GNU General Public License as published
+by the Free Software Foundation; or the Artistic License.
+
+See L<http://dev.perl.org/licenses/> for more information.
+
+=cut
+
 1;
