@@ -3,7 +3,7 @@ package MVC::Neaf::Route;
 use strict;
 use warnings;
 
-our $VERSION = '0.2800_01';
+our $VERSION = 0.23;
 
 =head1 NAME
 
@@ -27,7 +27,7 @@ use Scalar::Util qw( looks_like_number blessed );
 use URI::Escape qw( uri_unescape );
 
 use parent qw(MVC::Neaf::Util::Base);
-use MVC::Neaf::Util qw( canonize_path path_prefixes run_all run_all_nodie http_date make_getters );
+use MVC::Neaf::Util qw( canonize_path path_prefixes run_all run_all_nodie http_date );
 
 our @CARP_NOT = qw(MVC::Neaf MVC::Neaf::Request);
 
@@ -67,8 +67,6 @@ Route has the following read-only attributes:
 
 =item * hooks
 
-=item * helpers
-
 =back
 
 =cut
@@ -76,11 +74,10 @@ Route has the following read-only attributes:
 # Should just Moo here but we already have a BIG dependency footprint
 my @ESSENTIAL = qw( parent method path code );
 my @OPTIONAL  = qw(
-    param_regex path_info_regex strict
-    default helpers hooks
-    caller description public where
-    override tentative
-    cache_ttl
+    default cache_ttl
+    path_info_regex param_regex hooks
+    description public caller where tentative
+    override
 );
 my %RO_FIELDS;
 $RO_FIELDS{$_}++ for @ESSENTIAL, @OPTIONAL;
@@ -109,7 +106,7 @@ sub new {
         unless UNIVERSAL::isa($opt{code}, 'CODE');
     $class->my_croak("'public' endpoint must have a 'description'")
         if $opt{public} and not $opt{description};
-    $class->my_croak( "'default' must be unblessed hash" )
+    $class->_croak( "'default' must be unblessed hash" )
         if ref $opt{default} ne 'HASH';
     $class->my_croak("'method' must be a plain scalar")
         unless $opt{method} =~ /^[A-Z0-9_]+$/;
@@ -128,7 +125,7 @@ sub new {
     # preprocess regular expression for params
     if ( my $reg = $opt{param_regex} ) {
         my %real_reg;
-        $class->my_croak("'param_regex' must be a hash of regular expressions")
+        $class->_croak("param_regex must be a hash of regular expressions")
             if ref $reg ne 'HASH' or grep { !defined $reg->{$_} } keys %$reg;
         $real_reg{$_} = qr(^$reg->{$_}$)s
             for keys %$reg;
@@ -136,11 +133,12 @@ sub new {
     };
 
     if ( $opt{cache_ttl} ) {
-        $class->my_croak("'cache_ttl' must be a number")
+        $class->_croak("cache_ttl must be a number")
             unless looks_like_number($opt{cache_ttl});
         # as required by RFC
         $opt{cache_ttl} = -100000 if $opt{cache_ttl} < 0;
         $opt{cache_ttl} = $year if $opt{cache_ttl} > $year;
+        $opt{cache_ttl} = $opt{cache_ttl};
     };
 
     return bless \%opt, $class;
@@ -177,12 +175,20 @@ Check that route is locked.
 
 =cut
 
-# TODO 0.40 a version with croak
 sub is_locked {
     my $self = shift;
     return !!$self->{lock};
 };
 
+# TODO 0.30 -> Util::Base?
+sub _can_modify {
+    my $self = shift;
+    return unless $self->{lock};
+    # oops
+
+    croak "Modification of locked ".(ref $self)." attempted";
+};
+
 =head2 add_form()
 
     add_form( name => $validator )
@@ -254,7 +260,7 @@ my %FORM_ENGINE = (
 
 sub add_form {
     my ($self, $name, $spec, %opt) = @_;
-    # TODO 0.30 Make path-based?
+    # TODO 0.25 helper
 
     $name and $spec
         or $self->my_croak( "Form name and spec must be nonempty" );
@@ -295,13 +301,62 @@ sub get_form {
     # Aggressive caching for the win
     return $self->{forms}{$name} ||= do {
         my $parent = $self->parent;
-        croak("Failed to locate form '$name'")
+        $self->my_croak("Failed to locate form named $name")
             unless $parent;
         $parent->get_form($name);
     };
 };
 
-# TODO 0.40 get_view should be per-route, not global
+=head2 get_view
+
+=cut
+
+sub get_view {
+    my ($self, $name) = @_;
+
+    return $self->{views}{$name} ||= do {
+        my $parent = $self->parent;
+        $self->my_croak("Failed to locate view named $name")
+            unless $parent;
+        $parent->get_view($name);
+    };
+};
+
+=head2 append_defaults( \%hashref, \%hashref2 )
+
+Add more default values.
+
+=cut
+
+sub append_defaults {
+    my ($self, @hashes) = @_;
+
+    $self->_can_modify;
+
+    # merge hashes, older override newer
+    my %data = map { %$_ } grep { defined $_ }
+        ((reverse @hashes), $self->{default});
+
+    # kill undefs
+    defined $data{$_} or delete $data{$_}
+        for keys %data;
+
+    $self->{default} = \%data;
+    return $self;
+};
+
+=head2 set_hooks(\%phases)
+
+Install hooks. Currently no preprocessing is done.
+
+=cut
+
+sub set_hooks {
+    my ($self, $hooks) = @_;
+
+    # TODO 0.00 filter must be here
+    $self->{hooks} = $hooks;
+};
 
 =head2 post_setup
 
@@ -321,10 +376,44 @@ sub post_setup {
     my $neaf = $self->parent;
     # CALCULATE DEFAULTS
     # merge data sources, longer paths first
-    $self->{default} = $neaf->get_path_defaults ( $self->method, $self->path, $self->{default} );
-    $self->{hooks}   = $neaf->get_hooks   ( $self->method, $self->path );
-    $self->{helpers} = $neaf->get_helpers ( $self->method, $self->path );
+    my @sources = map { $neaf->{path_defaults}{$_} }
+        reverse path_prefixes( $self->path );
+    $self->append_defaults( @sources );
 
+    # CALCULATE HOOKS
+    # select ALL hooks prepared for upper paths
+    my $hook_tree = $neaf->{hooks}{ $self->method };
+    my @hook_by_path =
+        map { $hook_tree->{$_} || () } path_prefixes( $self->path );
+
+    # Merge callback stacks into one hash, in order
+    # hook = {method}{path}{phase}[nnn] => { code => sub{}, ... }
+    # We need to extract that sub {}
+    # We do so in a rather clumsy way that would short cirtuit
+    #     at all possibilities
+    # Premature optimization FTW!
+    my %phases;
+    foreach my $hook_by_phase (@hook_by_path) {
+        foreach my $phase ( keys %$hook_by_phase ) {
+            my $hook_list = $hook_by_phase->{$phase};
+            foreach my $hook (@$hook_list) {
+                # process excludes - if path starts with any, no go!
+                grep { $self->path =~ m#^\Q$_\E(?:/|$)# }
+                    @{ $hook->{exclude} }
+                        and next;
+                # TODO 0.90 filter out repetition
+                push @{ $phases{$phase} }, $hook->{code};
+                # TODO 0.30 also store hook info somewhere for better error logging
+            };
+        };
+    };
+
+    # the pre-reply, pre-cleanup should go in backward direction
+    # those are for cleaning up stuff
+    $phases{$_} and @{ $phases{$_} } = reverse @{ $phases{$_} }
+        for qw(pre_cleanup pre_reply);
+
+    $self->set_hooks( \%phases );
     $self->lock;
 
     return;
@@ -373,13 +462,19 @@ sub dispatch_logic {
 };
 
 # Setup getters
-make_getters( %RO_FIELDS );
+# TODO 0.30 Class::XSAccessors or smth
+foreach (keys %RO_FIELDS) {
+    my $method = $_;
+    my $sub = sub { $_[0]->{$method} };
+    no strict 'refs'; ## no critic
+    *{$method} = $sub;
+};
 
 =head1 LICENSE AND COPYRIGHT
 
 This module is part of L<MVC::Neaf> suite.
 
-Copyright 2016-2023 Konstantin S. Uvarin C<khedin@cpan.org>.
+Copyright 2016-2018 Konstantin S. Uvarin C<khedin@cpan.org>.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
