@@ -2,7 +2,7 @@ package MVC::Neaf::Util;
 
 use strict;
 use warnings;
-our $VERSION = 0.2203;
+our $VERSION = 0.23;
 
 =head1 NAME
 
@@ -22,6 +22,7 @@ use parent qw(Exporter);
 our @EXPORT_OK = qw(
     canonize_path http_date path_prefixes rex run_all run_all_nodie
     JSON encode_json decode_json
+    maybe_list
 );
 
 # use JSON::MaybeXS; # not now, see JSON() below
@@ -137,6 +138,29 @@ sub run_all_nodie {
     return $dead;
 };
 
+=head2 maybe_list
+
+    maybe_list( \$value, @defaults )
+
+If C<$value> is C<undef>, replace is with a copy of \@defaults.
+
+If C<$value> is a list, leave it as is.
+
+Otherwise, replace C<$value> with C<[ $value ]>.
+
+=cut
+
+sub maybe_list {
+    my ($scalref, @default) = @_;
+
+    if (ref $$scalref ne 'ARRAY') {
+        my $array = defined $$scalref ? [ my $tmp = $$scalref ] : \@default;
+        $$scalref = $array;
+    };
+
+    return $$scalref;
+};
+
 =head2 JSON()
 
 Because JSON::MaybeXS is not available on all systems, try to load it
@@ -164,15 +188,15 @@ if (!$luck) {
 
 =head1 LICENSE AND COPYRIGHT
 
-This module is part of the L<MVC::Neaf> suite.
+This module is part of L<MVC::Neaf> suite.
 
-Copyright 2016-2017 Konstantin S. Uvarin C<khedin@cpan.org>.
+Copyright 2016-2018 Konstantin S. Uvarin C<khedin@cpan.org>.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
 by the Free Software Foundation; or the Artistic License.
 
-See http://dev.perl.org/licenses/ for more information.
+See L<http://dev.perl.org/licenses/> for more information.
 
 =cut
 
